#    -*-    coding:    utf-8    -*-
import os

from Plugins.Plugin import PluginDescriptor

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch
from Components.config import config, configfile, getConfigListEntry, ConfigSubsection, ConfigYesNo, ConfigText, ConfigPassword, ConfigEnableDisable, ConfigSelection, NoSave, ConfigInteger
from Components.ConfigList import ConfigList, ConfigListScreen

from enigma import ePicLoad

# Plugin screens
from screen_main import RuyaMainScreen
from screen_section_movies import RuyaSectionMoviesScreen

import plugintools

from enigma import addFont
try:
    addFont('/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/fonts/Bariol_Bold.otf', 'Bariol', 100, 1)
    addFont('/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/fonts/ubuntul.otf', 'Ubuntu', 100, 1)
    addFont('/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/fonts/bebasneue.otf', 'Bebas', 100, 1)
except Exception as ex:
    print ex

config.plugins.ruya = ConfigSubsection()
config.plugins.ruya.pin = ConfigPassword(default = "")
config.plugins.ruya.show_adult = ConfigYesNo(default=False)
config.plugins.ruya.pin_adult = ConfigPassword(default = "")
config.plugins.ruya.server = ConfigText(default="", fixed_size=False)
config.plugins.ruya.username = ConfigText(default="", fixed_size=False)
config.plugins.ruya.password = ConfigText(default="", fixed_size=False)
config.plugins.ruya.cache_path = ConfigText(default="/media/hdd", fixed_size=False)
config.plugins.ruya.cache_enabled = ConfigEnableDisable(default=True)
config.plugins.ruya.install_dependencies = ConfigYesNo(default=False)
config.plugins.ruya.clean_cache = ConfigYesNo(default=False)
config.plugins.ruya.quality_selector = ConfigSelection(default="ask", choices = [("ask", "Ask"), ("sd", "Always SD"), ("hd", "Always HD")])
config.plugins.ruya.quality_selector_live = ConfigSelection(default="ask", choices = [("ask", "Ask"), ("sd", "Always SD"), ("hd", "Always HD")])
config.plugins.ruya.check_for_updates = ConfigYesNo(default=True)
config.plugins.ruya.last_movie_positions = ConfigText(default="")
config.plugins.ruya.last_episode_positions = ConfigText(default="")

def menu(menuid, **kwargs):
    if menuid == "mainmenu":
        return [("RuYa IPTV", main, "Where dreams come true", 4)]
    return []

def main(session, **kwargs):
    plugintools.loginfo("ruya.main Starting...")

    # Propagate settings to the plugintools module
    plugintools.set_setting("pin",config.plugins.ruya.pin.value)
    
    if config.plugins.ruya.show_adult.value:
        plugintools.set_setting("show_adult","true")
    else:
        plugintools.set_setting("show_adult","false")

    plugintools.set_setting("pin_adult",config.plugins.ruya.pin_adult.value)
    plugintools.set_setting("server",config.plugins.ruya.server.value)
    plugintools.set_setting("username",config.plugins.ruya.username.value)
    plugintools.set_setting("password",config.plugins.ruya.password.value)

    if config.plugins.ruya.cache_enabled.value:
        plugintools.set_setting("cache_enabled","true")
    else:
        plugintools.set_setting("cache_enabled","false")

    plugintools.set_setting("cache_path",config.plugins.ruya.cache_path.value)
    plugintools.set_setting("quality_selector",config.plugins.ruya.quality_selector.value)
    plugintools.set_setting("quality_selector_live",config.plugins.ruya.quality_selector_live.value)
    
    plugintools.set_setting("version","18")
    plugintools.set_setting("version_tag","1.1.8")

    if config.plugins.ruya.check_for_updates.value:
        plugintools.set_setting("check_for_updates","true")
    else:
        plugintools.set_setting("check_for_updates","false")

    # Create cache folder
    plugin_cache_folder = os.path.join( plugintools.get_setting("cache_path") , "ruya_cache" )
    plugintools.loginfo("ruya.main Creating cache folders in "+plugin_cache_folder)

    os.system("mkdir -p "+os.path.join( plugin_cache_folder ))

    # Check if Cache Folders were created successfully
    if not os.path.exists(plugin_cache_folder):
        plugintools.loginfo("ruya.main Can't create plugin cache folders")

    session.open(RuyaMainScreen)

def Plugins(**kwargs):

    return [
        PluginDescriptor(name="RuYa IPTV",description="Where dreams come true",where = PluginDescriptor.WHERE_MENU, fnc=menu),
        PluginDescriptor(name="RuYa IPTV",description="Where dreams come true",where = PluginDescriptor.WHERE_PLUGINMENU,icon="plugin.png",fnc=main),
        PluginDescriptor(name="RuYa IPTV",description="Where dreams come true",where = PluginDescriptor.WHERE_EXTENSIONSMENU,fnc=main),
    ]
