# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch

from enigma import ePicLoad
from enigma import eServiceReference

import plugintools
import api

from screen_custom_player import CustomPlayer

class RuyaBrowseLiveTVScreen(Screen):
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/browse-livetv.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, itemlist = [], screen_title=""):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.__init__")

        self.xy_to_item = dict()

        self.session = session

        if len(itemlist)>15:
            itemlist.insert(14,{"title":"More...","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nologo.png","action":"next_page"})

        self.itemlist = itemlist
        self.screen_title = screen_title
        Screen.__init__(self, session)

        i = 0
        # Setup widgets
        for row in range(0,3):
            for col in range(0,5):
                title = ""
                if i<len(self.itemlist):
                    title = self.itemlist[i]["title"]
                    thumbnail = self.itemlist[i]["thumbnail"]

                self["background"+str(row)+str(col)] = Label(_(" "))
                self["logo"+str(row)+str(col)] = Pixmap()
                self["title"+str(row)+str(col)] = Label(title)

                if i>=len(self.itemlist):
                    self["background"+str(row)+str(col)].hide()
                    self["logo"+str(row)+str(col)].hide()
                    self["title"+str(row)+str(col)].hide()

                self.xy_to_item[str(row)+str(col)] = i
                
                i = i + 1

        self["focus"] = Pixmap()
        self["screen_title"] = Label(screen_title)

        # Setup keys
        self["myActionMap"] = ActionMap(["WizardActions"],
        {
            "ok": self.action_ok,
            "back": self.action_back,
            "up": self.action_up,
            "down": self.action_down,
            "right": self.action_right,
            "left": self.action_left
        }, -1)

        self.active_row = 0;
        self.active_col = 0;

    def action_ok(self):
        index_of_item = self.xy_to_item[str(self.active_row)+str(self.active_col)]
        item = self.itemlist[index_of_item]
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_ok active=["+str(self.active_row)+","+str(self.active_col)+"] "+repr(item))

        if not "action" in item:
            '''
            self.hide()
            sref = eServiceReference(4097, 0, item["url"])
            self.session.nav.playService(sref)
            '''
            myreference = eServiceReference(4097,0,item["url"])
            myreference.setName(item["title"])
            self.session.open(CustomPlayer, myreference, item["title"] , "", item["thumbnail"] )

        elif item["title"]=="(Empty list)":
            self.action_back()

        elif item["action"]=="next_page":
            new_itemlist = self.itemlist[15:]
            self.session.open(RuyaBrowseLiveTVScreen,new_itemlist,"Live TV / All channels")

    def action_back(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_back")
        self.close(False,self.session)

    def action_up(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_up active=["+str(self.active_row)+","+str(self.active_col)+"]")

        if self.active_row>0:
            new_row = self.active_row - 1
        else:
            new_row = 2

        if self["logo"+str(new_row)+str(self.active_col)].visible:
            self.active_row = new_row
            self.move_focus()


    def action_down(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_down active=["+str(self.active_row)+","+str(self.active_col)+"]")

        if self.active_row<2:
            new_row = self.active_row + 1
        else:
            new_row = 0

        if self["logo"+str(new_row)+str(self.active_col)].visible:
            self.active_row = new_row
            self.move_focus()

    def action_right(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_right active=["+str(self.active_row)+","+str(self.active_col)+"]")
        if self.active_col<4:
            new_col = self.active_col + 1
        else:
            new_col = 0

        if self["logo"+str(self.active_row)+str(new_col)].visible:
            self.active_col = new_col
            self.move_focus()

    def action_left(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.action_left active=["+str(self.active_row)+","+str(self.active_col)+"]")
        if self.active_col>0:
            new_col = self.active_col - 1
        else:
            new_col = 4

        if self["logo"+str(self.active_row)+str(new_col)].visible:
            self.active_col = new_col
            self.move_focus()

    def move_focus(self):
        plugintools.loginfo("ruya.RuyaBrowseLiveTVScreen.move_focus active=["+str(self.active_row)+","+str(self.active_col)+"]")

        active_x,active_y = self["background"+str(self.active_row)+str(self.active_col)].getPosition()
        self["focus"].move(active_x-6,active_y-6)
