# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------
import os

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label,MultiColorLabel
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch

from enigma import ePicLoad
from enigma import eServiceReference
from enigma import eTimer
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api

from twisted.web.client import downloadPage

from screen_custom_player import CustomPlayer

class RuyaBrowseMoviesScreen(Screen):
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/browse-movies.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, itemlist = [], screen_title=""):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.__init__")

        self.xy_to_item = dict()

        self.session = session
        self.itemlist = itemlist
        self.screen_title = screen_title
        Screen.__init__(self, session)

        self.NUM_ROWS=3
        self.NUM_COLS=9

        i = 0
        # Setup widgets
        for row in range(0,self.NUM_ROWS):
            for col in range(0,self.NUM_COLS):
                title = ""
                if i<len(self.itemlist):
                    #title = self.itemlist[i]["title"]
                    thumbnail = self.itemlist[i]["thumbnail"]

                self["thumbnail"+str(row)+str(col)] = Pixmap()
                self["thumbnailfocus"+str(row)+str(col)] = Pixmap()
                self["thumbnailfocus"+str(row)+str(col)].hide()
                #self["title"+str(row)+str(col)] = Label(title)

                if i>=len(self.itemlist):
                    self["thumbnail"+str(row)+str(col)].hide()
                    #self["title"+str(row)+str(col)].hide()

                self.xy_to_item[str(row)+str(col)] = i
                
                i = i + 1

        self["thumbnailfocus00"].show()

        #self["focus"] = Pixmap()
        self["screen_title"] = Label(screen_title)
        self["title"] = Label("Previous ")

        self["quality_selector_background"] = Pixmap()
        self["quality_selector_sd_focus"] = Pixmap()
        self["quality_selector_hd_focus"] = Pixmap()
        self["quality_selector_sd_text"] = MultiColorLabel("Watch in SD")
        self["quality_selector_hd_text"] = MultiColorLabel("Watch in HD")

        self["continue_background"] = Label(" ")
        self["continue_no_focus"] = Pixmap()
        self["continue_yes_focus"] = Pixmap()
        self["continue_no_text"] = Label("Play from start")
        self["continue_yes_text"] = Label("Play from last point")
        self["continue_yes_focus"].hide()

        self["continue_background"].hide()
        self["continue_no_focus"].hide()
        self["continue_yes_focus"].hide()
        self["continue_no_text"].hide()
        self["continue_yes_text"].hide()
        self["continue_yes_focus"].hide()

        self["loading_background"]=Pixmap()
        self["loading_title"]=Label("Loading data...")
        
        self["loading_background"].hide()
        self["loading_title"].hide()

        # Setup keys
        self["myActionMap"] = ActionMap(["WizardActions"],
        {
            "ok": self.action_ok,
            "back": self.action_back,
            "up": self.action_up,
            "down": self.action_down,
            "right": self.action_right,
            "left": self.action_left
        }, -1)

        self.active_row = 0
        self.active_col = 0

        self.previous_active_row = 0
        self.previous_active_col = 0

        self.Scale = AVSwitch().getFramebufferScale()
        self.PicLoad = ePicLoad()
        self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)

        self.downloading_thumbnail_row = 0
        self.downloading_thumbnail_col = 0
        self.downloading_thumbnail_item_index = 0
        self.downloading_thumbnail_zoomed = False
        self.onLayoutFinish.append(self.loadNextThumbnail)

        self.dialog_opened = False
        self.dialog_selected = "sd"
        self.hide_dialog()

        self.Timer = eTimer()
        self.Timer_conn = self.Timer.timeout.connect(self.show_loader)
        self.loader_frame_number=1

    def loadNextThumbnail(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail")

        if not self.downloading_thumbnail_zoomed:
            self.downloading_thumbnail_item_index = self.downloading_thumbnail_item_index + 1
            if self.downloading_thumbnail_item_index > len(self.itemlist)-1:
                return

            self.downloading_thumbnail_col = self.downloading_thumbnail_col + 1
            if self.downloading_thumbnail_col > (self.NUM_COLS-1):
                self.downloading_thumbnail_col = 0
                self.downloading_thumbnail_row = self.downloading_thumbnail_row + 1

                if self.downloading_thumbnail_row > (self.NUM_ROWS-1):
                    self.downloading_thumbnail_col = 0
                    self.downloading_thumbnail_row = 0
                    return

        thumbnail = self.itemlist[self.downloading_thumbnail_item_index]["thumbnail"]
        thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail thumbnail="+thumbnail)
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail thumbnail_cache="+thumbnail_cache)

        if thumbnail_cache=="":

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+str(self.downloading_thumbnail_row)+","+str(self.downloading_thumbnail_col)+" to tmp "+thumbnail+"...")
            downloadPage(thumbnail, "/tmp/thumb").addCallback(self.thumbnail_loaded).addErrback(self.loadNextThumbnail)

        else:

            if os.path.exists(thumbnail_cache):
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Cached "+str(self.downloading_thumbnail_row)+","+str(self.downloading_thumbnail_col)+" "+thumbnail+"...")
                self.thumbnail_loaded(None)
            else:
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+str(self.downloading_thumbnail_row)+","+str(self.downloading_thumbnail_col)+" to cache "+thumbnail+"...")
                downloadPage(thumbnail, thumbnail_cache ).addCallback(self.thumbnail_loaded).addErrback(self.loadNextThumbnail)

    def thumbnail_loaded(self, picData):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.thumbnail_loaded")

        try:
            if self.downloading_thumbnail_zoomed:
                self.PicLoad.setPara([
                    self["thumbnailfocus"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.size().width(),
                    self["thumbnailfocus"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.size().height(),
                    self.Scale[0],
                    self.Scale[1],
                    0,
                    1,
                    "#002C2C39"])
            else:
                self.PicLoad.setPara([
                    self["thumbnail"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.size().width(),
                    self["thumbnail"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.size().height(),
                    self.Scale[0],
                    self.Scale[1],
                    0,
                    1,
                    "#002C2C39"])
            
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail StartDecode")

            thumbnail = self.itemlist[self.downloading_thumbnail_item_index]["thumbnail"]
            thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )
            if thumbnail_cache=="":
                thumbnail_cache="/tmp/thumb"

            self.PicLoad.startDecode(thumbnail_cache)
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail End")
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())
            self.loadNextThumbnail()

    def DecodePicture(self, PicInfo = ""):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.DecodePicture")
        ptr = self.PicLoad.getData()
        if self.downloading_thumbnail_zoomed:
            self["thumbnailfocus"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.setPixmap(ptr)
            self.downloading_thumbnail_zoomed = False
        else:
            self["thumbnail"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].instance.setPixmap(ptr)
            self.downloading_thumbnail_zoomed = True
        #self["title"+str(self.downloading_thumbnail_row)+str(self.downloading_thumbnail_col)].hide()

        #thumbnail_cached = os.path.join( "/mnt/hdd/ruya_cache/movies" , plugintools.get_md5( self.itemlist[self.downloading_thumbnail_item_index]["thumbnail"] ) )
        #import shutil
        #shutil.copyfile("/tmp/thumb", thumbnail_cached )

        self.loadNextThumbnail()

    def action_ok(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok active=["+str(self.active_row)+","+str(self.active_col)+"]")
        index_of_item = self.xy_to_item[str(self.active_row)+str(self.active_col)]
        item = self.itemlist[index_of_item]

        if self.dialog_opened:
            if self.dialog_selected == "sd":
                url = item["url"].split("|")[0]
            else:
                url = item["url"].split("|")[1]

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+url+"]")
            self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

        elif not "action" in item:
            url = item["url"]
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+url+"]")
            if "|" in url:
                if plugintools.get_setting("quality_selector")=="ask":
                    self.show_dialog()

                elif plugintools.get_setting("quality_selector")=="sd":
                    url = item["url"].split("|")[0]
                    self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

                elif plugintools.get_setting("quality_selector")=="hd":
                    url = item["url"].split("|")[1]
                    self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

            else:
                self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )
        
        elif item["action"]=="back":
            self.action_back()

        elif item["title"]=="(Empty list)":
            self.action_back()

        elif item["action"]=="tvshows_seasons":
            self.show_loader()
            tvshow = item["title"]
            api.get_tvshow_seasons(plugintools.get_setting("token") , tvshow , num_page = "0", per_page="12", callback = self.callback_after_api, action=item["action"], title="RUYA / TV SHOWS / "+tvshow.upper())

        elif item["action"] in ["movies_recent","movies_by_letter","movies_by_genre","tvshows_updated","tvshows_by_letter","tvshows_by_genre","movies_search_results","tvshows_search_results"]:
            self.show_loader()
            if "num_page" in item:
                num_page = item["num_page"]
            else:
                num_page = "0"

            if "per_page" in item:
                per_page = item["per_page"]
            else:
                per_page = "10"

            if item["action"]=="movies_recent":
                api.get_recent_movies(plugintools.get_setting("token") , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_by_letter":
                letter = item["extra"]
                api.get_movies_by_letter(plugintools.get_setting("token") , letter , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_by_genre":
                genre = item["extra"]
                api.get_movies_by_genre(plugintools.get_setting("token") , genre , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_search_results":
                self.hide_loader()
                terms = item["extra"]
                new_itemlist = api.movie_search(plugintools.get_setting("token") , terms , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
                self.session.open(RuyaBrowseMoviesScreen,new_itemlist,self.screen_title)
            elif item["action"]=="tvshows_updated":
                api.get_updated_tvshows(plugintools.get_setting("token") , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_by_letter":
                letter = item["extra"]
                api.get_tvshows_by_letter(plugintools.get_setting("token") , letter , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_by_genre":
                genre = item["extra"]
                api.get_tvshows_by_genre(plugintools.get_setting("token") , genre , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_search_results":
                self.hide_loader()
                terms = item["extra"]
                new_itemlist = api.tvshow_search(plugintools.get_setting("token") , terms , num_page = num_page, per_page = per_page)
                self.session.open(RuyaBrowseMoviesScreen,new_itemlist,self.screen_title)

    def callback_after_api(self,action,title,items):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.callback_after_api")

        self.hide_loader()

        try:
            if action in ["tvshows_seasons"]:
                from screen_section_tvshows import RuyaSectionTVShowsScreen
                self.session.open(RuyaSectionTVShowsScreen,items,title)
            else:
                self.session.open(RuyaBrowseMoviesScreen,items,title)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def show_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.show_loader")

        self["loading_background"].show()
        self["loading_title"].show()
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/loader-background-"+str(self.loader_frame_number)+".png")
        self["loading_background"].instance.setPixmap(pixmap)

        self.loader_frame_number = self.loader_frame_number + 1
        if self.loader_frame_number>8:
            self.loader_frame_number = 1

        self.Timer.start(250, False)

    def hide_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.hide_loader")

        self.Timer.stop()
        self["loading_background"].hide()
        self["loading_title"].hide()

    def action_back(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_back")

        if self.dialog_opened:
            self.hide_dialog()
        else:
            self.close(False,self.session)

    def action_up(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_up active=["+str(self.active_row)+","+str(self.active_col)+"]")

        if self.dialog_opened:
            self["quality_selector_sd_focus"].show()
            self["quality_selector_sd_text"].setForegroundColorNum(1)
            self["quality_selector_hd_focus"].hide()
            self["quality_selector_hd_text"].setForegroundColorNum(0)
            self.dialog_selected = "sd"
        else:
            if self.active_row>0:
                new_row = self.active_row - 1
            else:
                new_row = self.NUM_ROWS-1

            if self["thumbnail"+str(new_row)+str(self.active_col)].visible:
                self.active_row = new_row
                self.move_focus()

    def action_down(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_down active=["+str(self.active_row)+","+str(self.active_col)+"]")

        if self.dialog_opened:
            self["quality_selector_sd_focus"].hide()
            self["quality_selector_sd_text"].setForegroundColorNum(0)
            self["quality_selector_hd_focus"].show()
            self["quality_selector_hd_text"].setForegroundColorNum(1)
            self.dialog_selected = "hd"
        else:
            if self.active_row<(self.NUM_ROWS-1):
                new_row = self.active_row + 1
            else:
                new_row = 0

            if self["thumbnail"+str(new_row)+str(self.active_col)].visible:
                self.active_row = new_row
                self.move_focus()

    def action_right(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_right active=["+str(self.active_row)+","+str(self.active_col)+"]")
        if self.active_col<(self.NUM_COLS-1):
            new_col = self.active_col + 1
        else:
            new_col = 0

        if self["thumbnail"+str(self.active_row)+str(new_col)].visible:
            self.active_col = new_col
            self.move_focus()

    def action_left(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_left active=["+str(self.active_row)+","+str(self.active_col)+"]")
        if self.active_col>0:
            new_col = self.active_col - 1
        else:
            new_col = (self.NUM_COLS-1)

        if self["thumbnail"+str(self.active_row)+str(new_col)].visible:
            self.active_col = new_col
            self.move_focus()

    def move_focus(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.move_focus active=["+str(self.active_row)+","+str(self.active_col)+"]")

        #active_x,active_y = self["thumbnail"+str(self.active_row)+str(self.active_col)].getPosition()
        #self["focus"].move(active_x-6,active_y-6)

        self["thumbnailfocus"+str(self.previous_active_row)+str(self.previous_active_col)].hide()
        self["thumbnailfocus"+str(self.active_row)+str(self.active_col)].show()
        #self["thumbnailfocus"+str(self.active_row)+str(self.active_col)].scale()
        self.previous_active_row = self.active_row
        self.previous_active_col = self.active_col

        index_of_item = self.xy_to_item[str(self.active_row)+str(self.active_col)]
        item = self.itemlist[index_of_item]
        self["title"].setText(item["title"])

    def show_dialog(self):
        self.dialog_opened = True
        self["quality_selector_background"].show()
        self["quality_selector_sd_focus"].show()
        self["quality_selector_hd_focus"].hide()
        self["quality_selector_sd_text"].show()
        self["quality_selector_hd_text"].show()
        self["quality_selector_sd_text"].setForegroundColorNum(1)
        self["quality_selector_hd_text"].setForegroundColorNum(0)
        self.dialog_selected="sd"

    def hide_dialog(self):
        self.dialog_opened = False
        self["quality_selector_background"].hide()
        self["quality_selector_sd_focus"].hide()
        self["quality_selector_hd_focus"].hide()
        self["quality_selector_sd_text"].hide()
        self["quality_selector_hd_text"].hide()
        self.dialog_selected="sd"
