# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------

import os

from Screens.Screen import Screen
from Screens.InfoBarGenerics import InfoBarNotifications, InfoBarSeek, InfoBarShowHide, InfoBarAudioSelection, InfoBarCueSheetSupport, InfoBarSubtitleSupport
from Screens.AudioSelection import SubtitleSelection
from Screens.MessageBox import MessageBox

from Components.Label import Label
from Components.ActionMap import ActionMap
from Components.Pixmap import Pixmap
from Components.ServiceEventTracker import ServiceEventTracker
from Components.config import config, configfile

from enigma import eServiceReference
from enigma import eTimer
from enigma import iPlayableService

# For image loading
from Components.AVSwitch import AVSwitch
from enigma import ePicLoad
from twisted.web.client import downloadPage
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api

class CustomLiveTVPlayer(Screen, InfoBarNotifications, InfoBarSeek, InfoBarAudioSelection, InfoBarSubtitleSupport):
    STATE_IDLE = 0
    STATE_PLAYING = 1
    STATE_PAUSED = 2
    ENABLE_RESUME_SUPPORT = True
    ALLOW_SUSPEND = True

    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/player-live.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, service_list, service_index): #service, title, plot, thumbnail, background_color1="", background_color2=""):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__init__")

        Screen.__init__(self, session)
        InfoBarNotifications.__init__(self)

        InfoBarSeek.__init__(self)
        #InfoBarShowHide.__init__(self)
        InfoBarAudioSelection.__init__(self)
        InfoBarSubtitleSupport.__init__(self)
        #InfoBarCueSheetSupport.__init__(self)

        self.session = session
        self.service_list = service_list
        self.service_index = service_index

        mediaurl = self.service_list[self.service_index]["url"]
        if not mediaurl.startswith("http://"):
            mediaurl = api.get_livetv_url(plugintools.get_setting("token"),mediaurl)

        #mediaurl = "http://85.17.169.209:1935/live/JSC10w/playlist.m3u8"
        #plugintools.loginfo("ruya.CustomLiveTVPlayer.__init__ forced mediaurl="+mediaurl)

        service_reference = eServiceReference(4097,0,mediaurl)
        service_reference.setName(self.service_list[self.service_index]["title"])

        self.service = service_reference
        self.screen_timeout = 5000
        self.nextservice = None

        #self["background"]=Label(" ")
        self["thumbnail"]=Pixmap()
        title_split = self.service_list[self.service_index]["title"]
        if "(" in title_split:
            title_split = title_split.split("(")[0]
        self["title"]=Label( title_split )
        self["plot"]=Label( self.service_list[self.service_index]["plot"] )

        plugintools.loginfo("ruya.CustomLiveTVPlayer.__init__ evEOF=%d" % iPlayableService.evEOF)
        self.__event_tracker = ServiceEventTracker(screen = self, eventmap =
            {
                iPlayableService.evSeekableStatusChanged: self.__seekableStatusChanged,
                iPlayableService.evStart: self.__serviceStarted,
                iPlayableService.evBuffering: self.__serviceBuffering,
                iPlayableService.evEOF: self.__evEOF,
            })
        
        #self["actions"] = ActionMap(["OkCancelActions", "InfobarSeekActions", "MediaPlayerActions", "MovieSelectionActions", "MoviePlayerActions","ColorActions","NumberActions"],
        self["actions"] = ActionMap(["OkCancelActions", "InfobarSeekActions", "MediaPlayerActions", "MovieSelectionActions", "MoviePlayerActions","ColorActions","NumberActions","DirectionActions"],
        {
                "menu": self.menu,
                "ok": self.ok,
                "cancel": self.action_back,
                "stop": self.action_back,
                "pauseService": self.pauseService,
                "playpauseService": self.playpauseService,
                "showEventInfo": self.showVideoInfo,
                "subtitles": self.subtitleSelection,
                "seekFwd": self.seekFwd,
                "seekBack": self.seekBack,
                "up": self.action_up,
                "down": self.action_down,
                "channelUp": self.channel_up,
                "channelDown": self.channel_down,
                "5": self.reload,
            }, -2)

        self.hidetimer = eTimer()
        self.hidetimer_conn = self.hidetimer.timeout.connect(self.ok)
        self.returning = False

        self.state = self.STATE_PLAYING
        self.lastseekstate = self.STATE_PLAYING

        self.onPlayStateChanged = [ ]
        self.__seekableStatusChanged()
    
        self.play()
        self.onClose.append(self.__onClose)

        # For image loading
        self.thumbnail = self.service_list[self.service_index]["thumbnail"]
        self.Scale = AVSwitch().getFramebufferScale()
        self.PicLoad = ePicLoad()
        self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)
        self.onLayoutFinish.append(self.load_thumbnail)
        self.autoskip=False

        self.wakeup_timeout = 1000*60*60*4 # 4 hours
        self.wakeup_timer = eTimer()
        #self.wakeup_timer.timeout.get().append(self.wakeup_handler)
        self.wakeup_timer_conn = self.wakeup_timer.timeout.connect(self.wakeup_handler)
        self.wakeup_timer.start(self.wakeup_timeout)

    def wakeup_handler(self):
        self.session.openWithCallback( self.wakeup_answer , MessageBox , _("You have not moved for a long time... Are you sleeping?") , MessageBox.TYPE_YESNO, timeout=30, default=True)

    def wakeup_answer(self, result):
        if result:
            self.close()

    def action_up(self):

        if len(self.service_list)==1:
            return

        self.action_back()

    def action_down(self):
        
        if len(self.service_list)==1:
            return

        self.action_back()

    def reload(self):

        if len(self.service_list)==1:
            return

        if self.state != self.STATE_IDLE:
            self.stopCurrent()
        #self.play()

        self.load_channel()

    def channel_up(self):

        if len(self.service_list)==1:
            return

        if self.service_index>0:
            self.service_index = self.service_index - 1
        else:
            self.service_index = len(self.service_list)-1

        if self.service_list[self.service_index]["url"]=="":
            self.channel_up()
        else:
            self.load_channel()

    def channel_down(self):

        if len(self.service_list)==1:
            return

        if self.service_index < (len(self.service_list)-1):
            self.service_index = self.service_index + 1
        else:
            self.service_index = 0

        if self.service_list[self.service_index]["url"]=="":
            self.channel_down()
        else:
            self.load_channel()

    def load_channel(self):
        self.thumbnail = self.service_list[self.service_index]["thumbnail"]
        
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/thumb-black-info.png")
        self["thumbnail"].instance.setPixmap(pixmap)

        self.load_thumbnail()

        title_split = self.service_list[self.service_index]["title"]
        if "(" in title_split:
            title_split = title_split.split("(")[0]
        self["title"].setText( title_split )
        self["plot"].setText( self.service_list[self.service_index]["plot"] )

        self.showInfobar()

        mediaurl = self.service_list[self.service_index]["url"]
        if not mediaurl.startswith("http://"):
            mediaurl = api.get_livetv_url(plugintools.get_setting("token"),mediaurl)

        service_reference = eServiceReference(4097,0,mediaurl)
        service_reference.setName(self.service_list[self.service_index]["title"])

        self.service = service_reference
        self.play()

    def seekFwd(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.seekFwd")

        if len(self.service_list)>1:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.seekFwd hace channel_down")
            self.channel_down()
        else:
            try:
                service = self.session.nav.getCurrentService()
                seek = service and service.seek()
                if seek != None:
                    service_length = seek.getLength()[1]
                    plugintools.loginfo("ruya.CustomLiveTVPlayer.seekFwd service_length="+str(service_length))
                    skip_length = int( float( service_length ) / 30)
                    plugintools.loginfo("ruya.CustomLiveTVPlayer.seekFwd skip_length="+str(skip_length))

                    # five minutes
                    seek.seekRelative(1,skip_length)
            except:
                import traceback
                plugintools.loginfo(traceback.format_exc())

    def seekBack(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.seekBack")

        if len(self.service_list)>1:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.seekFwd hace channel_up")
            self.channel_up()
        else:

            try:
                service = self.session.nav.getCurrentService()
                seek = service and service.seek()
                if seek != None:
                    service_length = seek.getLength()[1]
                    plugintools.loginfo("ruya.CustomLiveTVPlayer.seekBack service_length="+str(service_length))
                    skip_length = int( float( service_length ) / 30)
                    plugintools.loginfo("ruya.CustomLiveTVPlayer.seekBack skip_length="+str(skip_length))

                    # five minutes
                    seek.seekRelative(-1,skip_length)
            except:
                import traceback
                plugintools.loginfo(traceback.format_exc())

    def skip_to_last_position(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.skip_to_last_position")

        current_movie_id = self.get_current_movie_id()

        if current_movie_id!="":
            # Leer posicion y mover el video
            last_movie_positions = ["030b4a92c9dec293d8715e9d8c27272c|17231009"] #config.plugins.ruya.last_movie_positions.value.split("#")
            plugintools.loginfo("ruya.CustomLiveTVPlayer.skip_to_last_position last_movie_positions="+repr(last_movie_positions))

            #"4212654368b7c623f414851c22af7a94|516930294"
            for last_movie_position in last_movie_positions:
                plugintools.loginfo("ruya.CustomLiveTVPlayer.skip_to_last_position last_movie_position="+repr(last_movie_position))
                movie_id_list = last_movie_position.split("|")[0]
                position_list = last_movie_position.split("|")[1]
                plugintools.loginfo("ruya.CustomLiveTVPlayer.skip_to_last_position movie_id_list="+movie_id_list+", position_list="+position_list)

                if current_movie_id == movie_id_list:
                    plugintools.loginfo("ruya.CustomLiveTVPlayer.skip_to_last_position we have position")
                    service = self.session.nav.getCurrentService()
                    seek = service and service.seek()
                    if seek != None:
                        seek.seekTo( long(position_list) )

    def save_last_position(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.save_last_position")

        service = self.session.nav.getCurrentService()
        seek = service and service.seek()
        if seek != None:
            service_length = seek.getLength()[1]
        else:
            service_length = 0

        plugintools.loginfo("ruya.CustomLiveTVPlayer.save_last_position service_length="+str(service_length))

        # Grabar posición
        if service_length!=0:
            current_movie_id = self.get_current_movie_id()
            config.plugins.ruya.last_movie_positions.value = current_movie_id+"|"+str(service_length)
            config.plugins.ruya.save()
            configfile.save()

    def get_current_movie_id(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.get_current_movie_id")
        current_movie_id = plugintools.find_single_match(self.service_list[self.service_index]["url"],"movie_id\=([a-z0-9]+)\&")
        plugintools.loginfo("ruya.CustomLiveTVPlayer.get_current_movie_id current_movie_id="+current_movie_id)
        return current_movie_id

    def load_thumbnail(self):
        thumbnail = self.thumbnail
        thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )

        if thumbnail_cache=="":

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+thumbnail+"...")
            downloadPage(thumbnail, "/tmp/thumb").addCallback(self.thumbnail_loaded)

        else:

            if os.path.exists(thumbnail_cache):
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Cached "+thumbnail+"...")
                self.thumbnail_loaded(None)
            else:
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+thumbnail+"...")
                downloadPage(thumbnail, thumbnail_cache ).addCallback(self.thumbnail_loaded)

    def thumbnail_loaded(self, picData):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.thumbnail_loaded")

        try:
            self.PicLoad.setPara([self["thumbnail"].instance.size().width(),self["thumbnail"].instance.size().height(),self.Scale[0],self.Scale[1],0,1,"#ff000000"])
            plugintools.loginfo("ruya.CustomLiveTVPlayer.thumbnail_loaded StartDecode")

            thumbnail = self.thumbnail
            thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )
            if thumbnail_cache=="":
                thumbnail_cache="/tmp/thumb"

            self.PicLoad.startDecode(thumbnail_cache)

            plugintools.loginfo("ruya.CustomLiveTVPlayer.thumbnail_loaded End")
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def DecodePicture(self, PicInfo = ""):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.DecodePicture")

        try:
            ptr = self.PicLoad.getData()
            self["thumbnail"].instance.setPixmap(ptr)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def action_back(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.action_back")
        self.close(False,self.session)

    def __onClose(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__onClose")
        self.session.nav.stopService()

    def __evEOF(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__evEOF evEOF=%d" % iPlayableService.evEOF)
        self.close(False,self.session)

    def __setHideTimer(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__setHideTimer")
        self.hidetimer.start(self.screen_timeout)

    def showInfobar(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.showInfobar")

        self.show()
        if self.state == self.STATE_PLAYING:
            self.__setHideTimer()
        else:
            pass

    def hideInfobar(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.hideInfobar")
        self.hide()
        self.hidetimer.stop()

    def ok(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.ok")
        if self.shown:
            self.hideInfobar()
        else:
            self.showInfobar()

    def showVideoInfo(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.showVideoInfo")
        self.showInfobar()

    def playService(self, newservice):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.playService")
        if self.state != self.STATE_IDLE:
            self.stopCurrent()
        self.service = newservice
        
        self.play()
        '''
        self.m3u8Console = eConsoleAppContainer()
        #self.m3u8Console.dataAvail.append(self.cbDataAvail)
        cmd = 'gst-launch playbin2 uri=' + url
        self.m3u8Console.execute(cmd)
        '''

    '''
    def doStreamAction(self):
        url = '/tmp/stream.avi'
        self.streamPlayerTimer.stop()
        sref = eServiceReference(4097, 0, url)
        self.session.nav.playService(sref)

    def cbDataAvail(self, data):
        print data
        if str(data) == 'Connected...':
            self.streamPlayerTimer = eTimer()
            self.streamPlayerTimer.timeout.get().append(self.doStreamAction)
            self.streamPlayerTimer.start(1000)
    '''

    def play(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.play")
        if self.state == self.STATE_PAUSED:
            if self.shown:
                self.__setHideTimer()   
        self.state = self.STATE_PLAYING
        self.session.nav.playService(self.service)
        if self.shown:
            self.__setHideTimer()

    def menu(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.menu")
        #self.skip_to_last_position()

    def stopCurrent(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.stopCurrent")
        self.session.nav.stopService()
        self.state = self.STATE_IDLE

    def playpauseService(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.playpauseService")
        if self.state == self.STATE_PLAYING:
            self.pauseService()
        elif self.state == self.STATE_PAUSED:
            self.unPauseService()

    def pauseService(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.pauseService")
        if self.state == self.STATE_PLAYING:
            self.setSeekState(self.STATE_PAUSED)
        
    def unPauseService(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.unPauseService")
        if self.state == self.STATE_PAUSED:
            self.setSeekState(self.STATE_PLAYING)

    def subtitleSelection(self):
        self.session.open(SubtitleSelection, self)
    
    def getSeek(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.getSeek")
        service = self.session.nav.getCurrentService()
        if service is None:
            return None

        seek = service.seek()

        if seek is None or not seek.isCurrentlySeekable():
            return None

        return seek

    def isSeekable(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.isSeekable")
        if self.getSeek() is None:
            return False
        return True

    def __seekableStatusChanged(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__seekableStatusChanged")
        if not self.isSeekable():
            self.setSeekState(self.STATE_PLAYING)
        else:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.__seekableStatusChanged seekable")

    def __serviceStarted(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__serviceStarted")
        self.state = self.STATE_PLAYING
        self.__seekableStatusChanged()

    def __serviceBuffering(self):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.__serviceBuffering")

        if not self.autoskip:
            self.autoskip=True
            #self.skip_to_last_position()

    def setSeekState(self, wantstate, dummy=False):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState")
        if wantstate == self.STATE_PAUSED:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState trying to switch to Pause- state:"+str(self.STATE_PAUSED))
        elif wantstate == self.STATE_PLAYING:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState trying to switch to playing- state:"+str(self.STATE_PLAYING))
        service = self.session.nav.getCurrentService()
        if service is None:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState No Service found")
            return False
        pauseable = service.pause()
        if pauseable is None:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState not pauseable.")
            self.state = self.STATE_PLAYING

        if pauseable is not None:
            plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState service is pausable")
            if wantstate == self.STATE_PAUSED:
                plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState WANT TO PAUSE")
                pauseable.pause()
                self.state = self.STATE_PAUSED
                if not self.shown:
                    self.hidetimer.stop()
                    self.show()
            elif wantstate == self.STATE_PLAYING:
                plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState WANT TO PLAY")
                pauseable.unpause()
                self.state = self.STATE_PLAYING
                if self.shown:
                    self.__setHideTimer()

        for c in self.onPlayStateChanged:
            plugintools.loginfo("c="+repr(c)+", state="+self.state)
            c(self.state)
        
        return True

    def doEofInternal(self, playing):
        plugintools.loginfo("ruya.CustomLiveTVPlayer.setSeekState")
        if not self.execing:
            return
        if not playing :
            return
        self.close()
