# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------

import os

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label,MultiColorLabel
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch

from enigma import ePicLoad
from enigma import eTimer
from twisted.web.client import downloadPage
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api
from updater import Updater

from screen_section_movies import RuyaSectionMoviesScreen
from screen_section_tvshows import RuyaSectionTVShowsScreen
from screen_section_livetv import RuyaSectionLiveTVScreen
from screen_settings import RuyaSettingsScreen

class RuyaMainScreen(Screen):
    # coordenadas: columna,fila
    # tamaños: ancho,alto
    # valign/halign: left, right, center
    # scrollbarMode: "showOnDemand", "showAlways", "showNever"
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/main.xml","r")
    skin = infile.read()
    infile.close()

    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, args = None):
        plugintools.loginfo("ruya.RuyaMainScreen.__init__")

        self.session = session
        Screen.__init__(self, session)

        # Setup widgets
        self["fanart"]=Pixmap()

        self["menu0"]=MultiColorLabel("MOVIES")
        self["menu1"]=MultiColorLabel("TV SHOWS")
        self["menu2"]=MultiColorLabel("LIVE TV")
        self["menu3"]=MultiColorLabel("SETTINGS")

        self["loading_background"]=Label(" ")
        self["loading_title"]=Label("Loading data...")
        self["loading_animation"]=Pixmap()
        
        self["loading_background"].hide()
        self["loading_title"].hide()
        self["loading_animation"].hide()

        # Setup keys
        self["myActionMap"] = ActionMap(["WizardActions"],
        {
            "ok": self.action_ok,
            "back": self.action_back,
            "right": self.action_right,
            "left": self.action_left
        }, -1)

        self.active = 0;

        # Saves current channel for restoring it on close
        self.previous_service = session.nav.getCurrentlyPlayingServiceReference()

        # Stops current channel
        self.session.nav.stopService()
        self.onLayoutFinish.append(self.login)

        self.Timer = eTimer()
        self.Timer_conn = self.Timer.timeout.connect(self.show_loader)
        self.loader_frame_number=1
        self.cover_backdrops = []

    def login(self):
        plugintools.loginfo("ruya.RuyaMainScreen.login")

        self.show_loader()

        self.cover_backdrops = api.get_cover_backdrops( plugintools.get_setting("server") )

        self.move_focus()

        # Perform login and save token
        token = api.login( plugintools.get_setting("server") , plugintools.get_setting("username") , plugintools.get_setting("password") )
        plugintools.set_setting("token",token)
        self.hide_loader()

        #if plugintools.get_setting("check_for_updates")=="true":
        #    updater = Updater(self.session)
        #    updater.check_for_updates()

    def show_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.show_loader")

        self["loading_background"].show()
        self["loading_title"].show()
        self["loading_animation"].show()
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/loader"+str(self.loader_frame_number)+".png")
        self["loading_animation"].instance.setPixmap(pixmap)

        self.loader_frame_number = self.loader_frame_number + 1
        if self.loader_frame_number>8:
            self.loader_frame_number = 1

        self.Timer.start(250, False)

    def hide_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.hide_loader")

        self.Timer.stop()
        self["loading_background"].hide()
        self["loading_title"].hide()
        self["loading_animation"].hide()

    def action_ok(self):
        plugintools.loginfo("ruya.RuyaMainScreen.action_ok")
        plugintools.loginfo("ruya.RuyaMainScreen.action_ok active="+str(self.active))

        if self.active==3:
            if plugintools.get_setting("pin")=="":
                self.session.open(RuyaSettingsScreen)
            else:
                self.session.openWithCallback(self.askForWord, InputBox, title=_("Enter PIN for settings"), text=" " * 55, maxSize=55, type=Input.TEXT)
        else:
            if plugintools.get_setting("token")=="":
                self.session.open(MessageBox,_("You need to enter the service credentials before using the plugin"), MessageBox.TYPE_INFO)
            else:
                if self.active==0:
                    itemlist = [ {"title":"Latest releases","action":"latest_releases"} , {"title":"Recent","action":"movies_recent"} , {"title":"A-Z","action":"movies_az"} , {"title":"Genres","action":"movies_genres"} , {"title":"Search","action":"movies_search"} ]
                    self.session.open(RuyaSectionMoviesScreen,itemlist,"RUYA / MOVIES")
                elif self.active==1:
                    itemlist = [ {"title":"Latest episodes","action":"tvshows_recent_episodes"} , {"title":"Updated shows","action":"tvshows_updated"} , {"title":"A-Z","action":"tvshows_az"} , {"title":"Genres","action":"tvshows_genres"} , {"title":"Search","action":"tvshows_search"} ]
                    self.session.open(RuyaSectionTVShowsScreen,itemlist,"RUYA / TV SHOWS")
                elif self.active==2:
                    itemlist = [ {"title":"All channels","action":"livetv_all"} , {"title":"Packages","action":"livetv_packages"} , {"title":"Categories","action":"livetv_genres"} ]
                    self.session.open(RuyaSectionLiveTVScreen,itemlist,"RUYA / LIVE TV")

    def askForWord(self, word):
        plugintools.loginfo("ruya.RuyaMainScreen.askForWord word="+str(word))
        if word is None:
            pass
        else:
            word = word.strip()
            if word==plugintools.get_setting("pin"):
                self.session.open(RuyaSettingsScreen)

    def action_back(self):
        plugintools.loginfo("ruya.RuyaMainScreen.action_back")
        
        # Restores the channel that user was watching when entering
        self.session.nav.playService(self.previous_service)

        # Closes the screen
        self.close(False,self.session)

    def action_right(self):
        plugintools.loginfo("ruya.RuyaMainScreen.action_right")

        self["menu"+str(self.active)].setForegroundColorNum(0)

        plugintools.loginfo("ruya.RuyaMainScreen.action_right active="+str(self.active))
        if self.active<3:
            self.active = self.active + 1
        else:
            self.active = 0

        self.move_focus()

    def action_left(self):
        plugintools.loginfo("ruya.RuyaMainScreen.action_left")

        self["menu"+str(self.active)].setForegroundColorNum(0)

        plugintools.loginfo("ruya.RuyaMainScreen.action_left active="+str(self.active))
        if self.active>0:
            self.active = self.active - 1
        else:
            self.active = 3

        self.move_focus()

    def move_focus(self):
        plugintools.loginfo("ruya.RuyaMainScreen.move_focus active"+str(self.active))

        #active_x,active_y = self["background"+str(self.active)].getPosition()
        #self["focus"].move(active_x-6,active_y-6)
        self["menu"+str(self.active)].setForegroundColorNum(1)

        if self.active==3:
            pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/fanart"+str(self.active)+".png")
            self["fanart"].instance.setPixmap(pixmap)
        else:
            self.load_backdrop()

    def load_backdrop(self):

        if len(self.cover_backdrops)>0:
            backdrop = self.cover_backdrops[int(self.active)]
            plugintools.loginfo("ruya.RuyaMainScreen.load_backdrop backdrop="+backdrop)
            cached_backdrop = plugintools.get_thumbnail_cache_folder( backdrop )
            plugintools.loginfo("ruya.RuyaMainScreen.load_backdrop cached_backdrop="+cached_backdrop)

            if cached_backdrop=="":

                plugintools.loginfo("ruya.RuyaMainScreen.load_backdrop Downloading "+backdrop+"...")
                downloadPage(backdrop, "/tmp/backdrop.png" ).addCallback(self.backdrop_loaded).addErrback(api.generic_errback)

            else:

                if os.path.exists(cached_backdrop+".png"):
                    plugintools.loginfo("ruya.RuyaMainScreen.load_backdrop Cached "+backdrop+"...")
                    self.backdrop_loaded(None)
                else:
                    plugintools.loginfo("ruya.RuyaMainScreen.load_backdrop Caching "+backdrop+"...")
                    downloadPage(backdrop, cached_backdrop+".png" ).addCallback(self.backdrop_loaded).addErrback(api.generic_errback)
        else:
            pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/fanart"+str(self.active)+".png")
            self["fanart"].instance.setPixmap(pixmap)

    def backdrop_loaded(self,data):
        plugintools.loginfo("ruya.RuyaMainScreen.backdrop_loaded")

        try:
            backdrop = self.cover_backdrops[int(self.active)]
            cached_backdrop = plugintools.get_thumbnail_cache_folder( backdrop )
            if cached_backdrop=="":
                cached_backdrop="/tmp/backdrop"
            cached_backdrop = cached_backdrop + ".png"
            plugintools.loginfo("ruya.RuyaMainScreen.backdrop_loaded cached_backdrop="+cached_backdrop)

            os.system("cp "+cached_backdrop+" /usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/fanart"+str(self.active)+".png")

            pixmap = LoadPixmap(cached_backdrop)
            self["fanart"].instance.setPixmap(pixmap)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())
