# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------
import os

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label,MultiColorLabel
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch

from enigma import ePicLoad
from enigma import eTimer
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api

from screen_browse_episodes import RuyaBrowseEpisodesScreen
from screen_browse_movies import RuyaBrowseMoviesScreen

class RuyaSectionMoviesScreen(Screen):
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/section-movies.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, itemlist = [], screen_title=""):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.__init__")

        self.session = session
        self.itemlist = itemlist
        self.screen_title = screen_title
        Screen.__init__(self, session)

        self.NUM_ITEMS_ON_SCREEN=14

        # Setup widgets
        for index in range(0,self.NUM_ITEMS_ON_SCREEN):

            #self["background_option"+str(index)] = Label(" ")
            if index < len(itemlist):
                self["title_option"+str(index)] = MultiColorLabel(itemlist[index]["title"])
            else:
                #self["background_option"+str(index)].hide()
                self["title_option"+str(index)] = MultiColorLabel(" ")
                self["title_option"+str(index)].hide()

            self["focus"+str(index)] = Pixmap()
            self["focus"+str(index)].hide()

        self["screen_title"] = Label(screen_title)
        self["scrollbar_section"] = Label(" ")
        self["scrollbar_pointer"] = Label(" ")

        self["loading_background"]=Pixmap()
        self["loading_title"]=Label("Loading data...")
        
        self["loading_background"].hide()
        self["loading_title"].hide()

        '''
        if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
            self["scrollbar_section"].show()
            self["scrollbar_pointer"].show()
        else:
            self["scrollbar_section"].hide()
            self["scrollbar_pointer"].hide()
        '''
        self["scrollbar_section"].hide()
        self["scrollbar_pointer"].hide()

        # Setup keys
        self["myActionMap"] = ActionMap(["WizardActions"],
        {
            "ok": self.action_ok,
            "back": self.action_back,
            "up": self.action_up,
            "down": self.action_down,
            "left": self.action_left,
            "right": self.action_right
        }, -1)

        self.active = 0
        self.first_item_on_screen = 0

        if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
            self.MAX_ITEM=self.NUM_ITEMS_ON_SCREEN-1
        else:
            self.MAX_ITEM=len(self.itemlist)-1

        self.Timer = eTimer()
        self.Timer_conn = self.Timer.timeout.connect(self.show_loader)
        self.loader_frame_number=1
        self.onLayoutFinish.append(self.move_focus)        

    def callback_after_api(self,action,title,items):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.callback_after_api")

        self.hide_loader()

        try:
            if action in ["movies_recent","latest_releases","movies_by_letter","movies_by_genre"]:
                self.session.open(RuyaBrowseMoviesScreen,items,title)
            elif action in ["movies_genres"]:
                self.session.open(RuyaSectionMoviesScreen,items,title)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def action_ok(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_ok")
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_ok active="+str(self.active))

        item = self.itemlist[ self.first_item_on_screen + self.active ]

        if item["title"]=="(Empty list)":
            self.action_back()

        elif item["action"]=="movies_recent":
            self.show_loader()
            api.get_recent_movies(plugintools.get_setting("token") , num_page = "0", per_page = str(self.NUM_ITEMS_ON_SCREEN-1), callback = self.callback_after_api, action=item["action"], title="RUYA / MOVIES / RECENT")

        elif item["action"]=="latest_releases":
            self.show_loader()
            api.get_latest_releases(plugintools.get_setting("token") , num_page = "0", per_page = str(self.NUM_ITEMS_ON_SCREEN-1), callback = self.callback_after_api, action=item["action"], title="RUYA / MOVIES / LATEST RELEASES")

        elif item["action"]=="movies_az":
            new_itemlist = []
            for letter in ['0-9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z']:
                new_itemlist.append({"title":letter,"action":"movies_by_letter"})
            self.session.open(RuyaSectionMoviesScreen,new_itemlist,"RUYA / MOVIES / A-Z")

        elif item["action"]=="movies_genres":
            self.show_loader()
            api.get_movie_genres(plugintools.get_setting("token"), callback = self.callback_after_api, action=item["action"], title="RUYA / MOVIES / GENRES")

        elif item["action"]=="movies_by_letter":
            self.show_loader()
            letter = item["title"]
            api.get_movies_by_letter(plugintools.get_setting("token"),letter , num_page = "0", per_page=str(self.NUM_ITEMS_ON_SCREEN-1), callback = self.callback_after_api, action=item["action"], title="RUYA / MOVIES / A-Z / STARTING WITH "+letter)

        elif item["action"]=="movies_by_genre":
            self.show_loader()
            genre = item["title"]
            api.get_movies_by_genre(plugintools.get_setting("token"),genre , num_page = "0", per_page=str(self.NUM_ITEMS_ON_SCREEN-1), callback = self.callback_after_api, action=item["action"], title="RUYA / MOVIES / GENRES / "+genre.upper())

        elif item["action"]=="movies_search":
            self.session.openWithCallback(self.askForWord, InputBox, title=_("Search term..."), text=" " * 55, maxSize=55, type=Input.TEXT)

    def show_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.show_loader")

        self["loading_background"].show()
        self["loading_title"].show()
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/loader-background-"+str(self.loader_frame_number)+".png")
        self["loading_background"].instance.setPixmap(pixmap)

        self.loader_frame_number = self.loader_frame_number + 1
        if self.loader_frame_number>8:
            self.loader_frame_number = 1

        self.Timer.start(70, False)

    def hide_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.hide_loader")

        self.Timer.stop()
        self["loading_background"].hide()
        self["loading_title"].hide()

    def askForWord(self, word):
        plugintools.loginfo("ruya.RuyaMainScreen.askForWord word="+str(word))
        if word is None:
            pass
        else:
            word = word.strip()
            new_itemlist = api.movie_search(plugintools.get_setting("token"), word , num_page = "0", per_page="25")
            if len(new_itemlist)==25:
                new_itemlist.append({"title":"More...","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"movies_search_results","extra":word,"num_page":"1","per_page":"10"})
            self.session.open(RuyaBrowseMoviesScreen,new_itemlist,"RUYA / MOVIES / SEARCH / "+word.upper())

    def action_back(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_back")
        self.close(False,self.session)

    def scroll_up(self):

        if (self.first_item_on_screen+self.NUM_ITEMS_ON_SCREEN) < len(self.itemlist):
            self.first_item_on_screen = self.first_item_on_screen + 1
            self.repaint_list()

    def scroll_down(self):

        if self.first_item_on_screen>0:
            self.first_item_on_screen = self.first_item_on_screen - 1
            self.repaint_list()

    def repaint_list(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.repaint_list")

        for index_on_screen in range(0,self.NUM_ITEMS_ON_SCREEN):
            index_on_itemlist = index_on_screen + self.first_item_on_screen

            if index_on_itemlist < len(self.itemlist):
                self["title_option"+str(index_on_screen)].setText( self.itemlist[index_on_itemlist]["title"] )
            else:
                self["title_option"+str(index_on_screen)].setText( "" )

        self.update_scrollbar_pointer()

    def action_up(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_up active="+str(self.active))

        #self.hide_loader()

        if self.active>0:
            self.active = self.active - 1
            self.move_focus()
        else:
            if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
                self.scroll_down()
            else:
                self.active = self.MAX_ITEM
                self.move_focus()

        self.update_scrollbar_pointer()

    def action_down(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_down active="+str(self.active))

        if self.active<self.MAX_ITEM:
            self.active = self.active + 1
            self.move_focus()
        else:
            if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
                self.scroll_up()
            else:
                self.active = 0
                self.move_focus()

        self.update_scrollbar_pointer()

    def action_left(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.first_item_on_screen = self.first_item_on_screen - self.NUM_ITEMS_ON_SCREEN
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        if self.first_item_on_screen<0:
            self.first_item_on_screen = 0
            self.active = 0
            plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.repaint_list()
        self.move_focus()

    def action_right(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.first_item_on_screen = self.first_item_on_screen + self.NUM_ITEMS_ON_SCREEN
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        if (self.first_item_on_screen + self.NUM_ITEMS_ON_SCREEN) > len(self.itemlist):

            self.first_item_on_screen = len(self.itemlist) - self.NUM_ITEMS_ON_SCREEN
            if self.first_item_on_screen < 0:
                self.first_item_on_screen = 0

            self.active = self.NUM_ITEMS_ON_SCREEN-1
            if self.active>len(self.itemlist)-1:
                self.active = len(self.itemlist)-1

            plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.repaint_list()
        self.move_focus()

    def update_scrollbar_pointer(self):

        if self["scrollbar_section"].visible:
            # Get scrollbar coordinates and size
            scrollbar_x , scrollbar_y = self["scrollbar_section"].getPosition()
            plugintools.loginfo("scrollbar_x="+str(scrollbar_x)+", scrollbar_y="+str(scrollbar_y))

            scrollbar_height = 510

            # Get current item on itemlist
            index_on_itemlist = self.active + self.first_item_on_screen
            plugintools.loginfo("index_on_itemlist="+str(index_on_itemlist))

            # Calculates pointer coordinates
            step_y = scrollbar_height // len(self.itemlist)
            plugintools.loginfo("step_y="+str(step_y))

            pointer_x = scrollbar_x
            pointer_y = scrollbar_y + (step_y * index_on_itemlist)
            plugintools.loginfo("pointer_x="+str(pointer_x)+", pointer_y="+str(pointer_y))

            self["scrollbar_pointer"].move(pointer_x,pointer_y)

    def move_focus(self):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.move_focus active="+str(self.active))

        #active_x,active_y = self["background_option"+str(self.active)].getPosition()
        #self["focus"].move(active_x-6,active_y-6)
        for i in range(0,self.NUM_ITEMS_ON_SCREEN):
            if i==self.active:
                self["focus"+str(i)].show()
                self["title_option"+str(i)].setForegroundColorNum(1)
            else:
                self["focus"+str(i)].hide()
                self["title_option"+str(i)].setForegroundColorNum(0)
