# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox

from Components.Label import Label
from Components.ActionMap import ActionMap
from Components.Pixmap import Pixmap
from Components.config import config, configfile, getConfigListEntry, ConfigSubsection, ConfigYesNo, ConfigText, ConfigPassword, ConfigEnableDisable, ConfigSelection, NoSave, ConfigInteger
from Components.ConfigList import ConfigList, ConfigListScreen

from enigma import eTimer
from enigma import eConsoleAppContainer
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api
import os

class RuyaSettingsScreen(Screen, ConfigListScreen):
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/settings.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session):
        Screen.__init__(self, session)

        self["actions"] = ActionMap(["SetupActions", "ColorActions"],
        {
            "ok": self.keySave,
            "green": self.keySave,
            "red": self.keyCancel,
            "cancel": self.keyCancel
        }, -2)

        self["loading_background"]=Label(" ")
        self["loading_title"]=Label("Update progress")
        self["loading_animation"]=Pixmap()
        
        self["loading_background"].hide()
        self["loading_title"].hide()
        self["loading_animation"].hide()

        self["version"] = Label( "Version "+plugintools.get_setting("version_tag"))
        self["screen_title"] = Label( "Settings " )

        self.cfglist = []
        self.cfglist.append(getConfigListEntry(_("Server name:"), config.plugins.ruya.server))
        self.cfglist.append(getConfigListEntry(_("Username:"), config.plugins.ruya.username))
        self.cfglist.append(getConfigListEntry(_("Password:"), config.plugins.ruya.password))
        self.cfglist.append(getConfigListEntry(_("Enable thumbnail caching:"), config.plugins.ruya.cache_enabled))
        self.cfglist.append(getConfigListEntry(_("Thumbnail cache folder:"), config.plugins.ruya.cache_path))
        self.cfglist.append(getConfigListEntry(_("Install dependencies:"), config.plugins.ruya.install_dependencies))
        self.cfglist.append(getConfigListEntry(_("Clean thumbnail cache:"), config.plugins.ruya.clean_cache))
        self.cfglist.append(getConfigListEntry(_("Preferred VOD quality:"), config.plugins.ruya.quality_selector))
        self.cfglist.append(getConfigListEntry(_("Preferred Live TV quality:"), config.plugins.ruya.quality_selector_live))
        #self.cfglist.append(getConfigListEntry(_("Check for updates:"), config.plugins.ruya.check_for_updates))
        self.cfglist.append(getConfigListEntry(_("PIN for settings:"), config.plugins.ruya.pin))
        self.cfglist.append(getConfigListEntry(_("Show adult content:"), config.plugins.ruya.show_adult))
        self.cfglist.append(getConfigListEntry(_("PIN for adult content:"), config.plugins.ruya.pin_adult))
        ConfigListScreen.__init__(self, self.cfglist, session)

        self.Timer = eTimer()
        self.Timer_conn = self.Timer.timeout.connect(self.show_loader)
        self.loader_frame_number=1

        self.install_commands = []
        self.install_commands.append(["Updating package list","opkg update"])
        self.install_commands.append(["Checking package python-distutils","opkg install python-distutils"])
        self.install_commands.append(["Checking package python-compression","opkg install python-compression"])
        self.install_commands.append(["Checking package python-ctypes","opkg install python-ctypes"])
        self.install_commands.append(["Checking package python-zlib","opkg install python-zlib"])
        self.install_commands.append(["Checking package python-subprocess","opkg install python-subprocess"])
        self.install_commands.append(["Checking package python-pkgutil","opkg install python-pkgutil"])
        #self.install_commands.append(["Checking package python-json","opkg install python-json"])
        self.install_commands.append(["Checking package python-textutils","opkg install python-textutils"])
        self.install_commands.append(["Checking package python-shell","opkg install python-shell"])
        self.install_commands.append(["Checking package python-io","opkg install python-io"])
        self.install_commands.append(["Checking package python-io","opkg install python-io"])
        self.install_commands.append(["Checking package gst-plugin-flv","opkg install gst-plugin-flv"])
        self.install_commands.append(["Checking package gst-plugin-rtmp","opkg install gst-plugin-rtmp"])
        self.install_commands.append(["Checking package gst-plugin-matroska","opkg install gst-plugin-matroska"])
        self.install_commands.append(["Checking package gst-plugin-fragmented","opkg install gst-plugin-fragmented"])

        self.install_command_index = 0
        self.container=None

    def keySave(self):
        config.plugins.ruya.save()
        configfile.save()

        # Propagate settings to the plugintools module
        plugintools.set_setting("pin",config.plugins.ruya.pin.value)
        if config.plugins.ruya.show_adult.value:
            plugintools.set_setting("show_adult","true")
        else:
            plugintools.set_setting("show_adult","false")
        plugintools.set_setting("pin_adult",config.plugins.ruya.pin_adult.value)
        plugintools.set_setting("server",config.plugins.ruya.server.value)
        plugintools.set_setting("username",config.plugins.ruya.username.value)
        plugintools.set_setting("password",config.plugins.ruya.password.value)
        
        if config.plugins.ruya.cache_enabled.value:
            plugintools.set_setting("cache_enabled","true")
        else:
            plugintools.set_setting("cache_enabled","false")
        plugintools.set_setting("cache_path",config.plugins.ruya.cache_path.value)
        plugintools.set_setting("quality_selector",config.plugins.ruya.quality_selector.value)
        plugintools.set_setting("quality_selector_live",config.plugins.ruya.quality_selector_live.value)

        #if config.plugins.ruya.check_for_updates.value:
        #    plugintools.set_setting("check_for_updates","true")
        #else:
        #    plugintools.set_setting("check_for_updates","false")

        # Perform login and save new token
        token = api.login( plugintools.get_setting("server") , plugintools.get_setting("username") , plugintools.get_setting("password") )
        plugintools.set_setting("token",token)

        # Install dependencies
        if config.plugins.ruya.clean_cache.value:
            try:

                plugin_cache_folder = os.path.join( config.plugins.ruya.cache_path.value , "ruya_cache" )
                plugintools.loginfo("Removing cache folder "+plugin_cache_folder)

                self.show_loader()
                self["loading_title"].setText( "Cleaning cache..." )

                # Removes full dir
                #import shutil
                #shutil.rmtree(config.plugins.ruya.cache_path.value)
                os.system("rm -rf "+os.path.join( plugin_cache_folder ))


                # Create it again
                plugintools.loginfo("Creating it again")
                os.system("mkdir -p "+os.path.join( plugin_cache_folder ))
                
                # Logs success
                plugintools.loginfo("Done!")
                config.plugins.ruya.clean_cache.value = False
                config.plugins.ruya.save()
                configfile.save()
            except:
                plugintools.loginfo("Cannot remove cache folder")

        if config.plugins.ruya.install_dependencies.value:
            plugintools.loginfo("Installing dependencies")
            config.plugins.ruya.install_dependencies.value = False
            config.plugins.ruya.save()
            configfile.save()

            self.show_loader()
            self.container=eConsoleAppContainer()
            self.container_appClosed_conn = self.container.appClosed.connect(self.executed_command)
            self.executed_command(0)
        else:
            self.session.open(MessageBox,_("The changes has been saved"), MessageBox.TYPE_INFO)
            self.close()

    def executed_command(self,retval):
        plugintools.loginfo("executed_command")

        if (self.install_command_index < len(self.install_commands)):
            install_command = self.install_commands[self.install_command_index]
            plugintools.loginfo("New command '"+install_command[0]+"' as '"+install_command[1]+"'")
            self.install_command_index = self.install_command_index + 1
            self["loading_title"].setText( install_command[0] )
            self.container.execute( install_command[1] )
        else:
            self.session.openWithCallback(self.confirmation_callback, MessageBox, "Dependencies has been installed and your settings has been saved. Do you want to restart now?", MessageBox.TYPE_YESNO)
            self.close()

    def confirmation_callback(self,result):

        if result:
            self.container.execute( "reboot" )

    def keyCancel(self):
        for item in self.cfglist:
            item[1].cancel()
        self.session.open(MessageBox,_("The changes has not been saved"), MessageBox.TYPE_INFO)
        self.close()

    def show_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.show_loader")

        self["loading_background"].show()
        self["loading_title"].show()
        self["loading_animation"].show()
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/loader"+str(self.loader_frame_number)+".png")
        self["loading_animation"].instance.setPixmap(pixmap)

        self.loader_frame_number = self.loader_frame_number + 1
        if self.loader_frame_number>8:
            self.loader_frame_number = 1

        self.Timer.start(250, False)

    def hide_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.hide_loader")

        self.Timer.stop()
        self["loading_background"].hide()
        self["loading_title"].hide()
        self["loading_animation"].hide()
